/*
   Copyright (C) 2003 Christopher Yeoh <cyeoh@samba.org>
   Copyright (C) 2003 Geoffrey D. Bennett <g@netcraft.com.au>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA. 
*/
#ifndef __TC_CLIENT_H
#define __TC_CLIENT_H

/* Move def */
#define TC_MOVE_FIRST_MOVE 1
#define TC_MOVE_LEFT 1
#define TC_MOVE_RIGHT 2
#define TC_ROTATE_CLOCKWISE 3
#define TC_ROTATE_ANTICLOCKWISE 4
#define TC_MOVE_DOWN 5
#define TC_MOVE_DROP 6
#define TC_MOVE_LAST_MOVE 6

/* Field dimensions */
#define TC_FIELD_WIDTH 12
#define TC_FIELD_HEIGHT 22
#define TC_FIELD_SIZE (TC_FIELD_WIDTH*TC_FIELD_HEIGHT)

/* Special definitions */
#define TC_SPECIAL_FIRST_SPECIAL 6
#define TC_SPECIAL_ADD_LINE 6
#define TC_SPECIAL_CLEAR_SPECIALS 11
#define TC_SPECIAL_CLEAR_LINE 7
#define TC_SPECIAL_BLOCK_GRAVITY 12
#define TC_SPECIAL_NUKE 8
#define TC_SPECIAL_BLOCK_BOMB 14
#define TC_SPECIAL_BLOCK_QUAKE 13
#define TC_SPECIAL_CLEAR_RANDOM 9
#define TC_SPECIAL_SWITCH_FIELDS 10
#define TC_SPECIAL_LAST_SPECIAL 14

/* These are not really "specials" */
#define TC_SPECIAL_ADDALL1 15
#define TC_SPECIAL_ADDALL2 16
#define TC_SPECIAL_ADDALL4 17


/* Block type definitions

TC_BLOCK_O
.xx.
.xx.
....
....
 	 
 	 
TC_BLOCK_I_1 2
xxxx
....
....
....
 	 
 	 
TC_BLOCK_I_2 3
..x.
..x.
..x.
..x.
 	 
 	 
TC_BLOCK_S_1 4
..xx
.xx.
....
....
	 
	 
TC_BLOCK_S_2 5
.x..
.xx.
..x.
....
	 
	 
TC_BLOCK_Z_1 6
.xx.
..xx
....
....
		
		
TC_BLOCK_Z_2 7
..x.
.xx.
.x..
....
	
	
TC_BLOCK_L_1 8
.xxx
.x..
....
....
	
	
TC_BLOCK_L_2 9
.x..
.x..
.xx.
....
	 
	 
TC_BLOCK_L_3 10
...x
.xxx
....
....
	
	
TC_BLOCK_L_4 11
.xx.
..x.
..x.
....
	 
	 
TC_BLOCK_J_1 12
.xxx
...x
....
....
		
		
TC_BLOCK_J_2 13
.xx.
.x..
.x..
....
	
	
TC_BLOCK_J_3 14
.x..
.xxx
....
....
		
		
TC_BLOCK_J_4 15
..x.
..x.
.xx.
....
	
	
TC_BLOCK_T_1 16
.xxx
..x.
....
....
	 
	 
TC_BLOCK_T_2 17
.x..
.xx.
.x..
....
	
	
TC_BLOCK_T_3 18
..x.
.xxx
....
....
	 	
	 	
TC_BLOCK_T_4 19
..x.
.xx.
..x.
....

*/

#define TC_BLOCK_NONE 0
#define TC_BLOCK_O 1
#define TC_BLOCK_I_1 2
#define TC_BLOCK_I_2 3
#define TC_BLOCK_S_1 4
#define TC_BLOCK_S_2 5
#define TC_BLOCK_Z_1 6
#define TC_BLOCK_Z_2 7
#define TC_BLOCK_L_1 8
#define TC_BLOCK_L_2 9
#define TC_BLOCK_L_3 10
#define TC_BLOCK_L_4 11
#define TC_BLOCK_J_1 12
#define TC_BLOCK_J_2 13
#define TC_BLOCK_J_3 14
#define TC_BLOCK_J_4 15
#define TC_BLOCK_T_1 16
#define TC_BLOCK_T_2 17
#define TC_BLOCK_T_3 18
#define TC_BLOCK_T_4 19
/* Rotating a block will (with the exception of 'O' block
	 effectively transition it to another a block type. This
	 type of representation removes the need for the concept of
   orientation */

struct TC_GameData;
struct TC_PlayerData;

/* Connect */
int connectToServer(const char *Hostname, int Port, const char *Username,
										char *TeamName, char *Channel);
/* Pass empty string as TeamName if not member of a team */
/* Pass empty string as Channel if you don't care */
/* 0 if sucessfully connected, non zero otherwise */

/* Start */
int Start(struct TC_GameData **Data);
/* 0 if game has started, non zero otherwise (eg error)
	 Call will block until game starts
	 Start should be called again as soon as you die if you
	 wish to be notified when the next game starts
	 Data will stay constant for duration of game */

/* Error string */
extern char *tc_error_string;

/* Returns 0 if valid move */
int MoveBlock(int MoveCommand);
/* Note move may also no longer be valid if the field has changed
	 since you calculated what you should do. */

/* Use special */
int UseSpecial(int PlayerNum);
/* Returns 0 if ok
	 1 otherwise (eg target player is no longer playing) */

/* Discard the special at the head of the list */
void DiscardSpecial(void);

/* Get field for player */
char *GetField(int PlayerNum);
/* Returns NULL if player is dead
	 Its up to you to free this memory

	 Note that the field returned is a 2 dimensional array
	 field[TC_FIELD_WIDTH][TC_FIELD_HEIGHT]

	 field[0][0] is the top left corner,
	 field[TC_FIELD_WIDTH-1][TC_FIELD_HEIGHT-1] is the bottom right corner
 */

/* Get list of specials */
void GetSpecials(char *Specials);
/* Null terminated array of specials */

/* Get current block dropping */
int GetCurrentBlock(int *BlockType, int *XPosition, int *YPosition);
/* Note that BlockType will be TC_BLOCK_NONE when no block
	 is dropping. This will be the case for approximately a second
	 after the previous block has finished falling. In this case
   the value of XPosition and YPosition is undefined 

	 XPosition and YPosition refers to the top left corner
	 of a 4x4 block as defined above.

	 Returns 0 if ok, non-zero if game over
    (1 if player lost, 2 if player won)
*/


/* Get next block to drop */
void GetNextBlock(int *BlockType);

/* Messages */
int GetMessage(int *FromPlayer, char **Message);
/* Normally out of band, but in this case you will need to poll for
	 this information. One line returned each time.

	 Returns 0 on sucess, 1 if there is no message waiting.
	 
	 You should free the string returned
*/

/* Send a message */
void SendMessage(int ToPlayer, char *Message);
/* ToPlayer should be 0 if you want to send it to everyone */

/* Gets the next use (in order) of special on another player */
int GetSpecialNotification(int *FromPlayer, int *ToPlayer, int *Special);
/* Returns 0 of there was use of special, non-zero otherwise.
 Note that ToPlayer will be -1 if special was sent to everyone (eg
 add to all even when someone completes more than 1 line)
 Note that you're not informed of your own use of specials  */



/* Access Game data */

/* Get Player Information */
int GetNumPlayers(struct TC_GameData *Data);
struct TC_PlayerData *GetPlayer(struct TC_GameData *Data, int Player);
/* Don't free this! */

/* Maximum number of specials you can store */
int GetMaxSpecials(struct TC_GameData *Data);

/* Get Own PlayerNumber */
int GetOwnPlayerNumber(struct TC_GameData *Data);

/* Free memory allocated for game data by Start call */
void FreeGameData(struct TC_GameData *Data);


/* Get Player data */
int GetPlayerNumber(struct TC_PlayerData *Player);
char *GetPlayerName(struct TC_PlayerData *Player);
char *GetPlayerTeam(struct TC_PlayerData *Player);

/* Versions of all the commands we can use during a game that might
 * block on reading from tetribot.
 *
 * Call txWhatever() to send the command to tetribot.
 * Call rxWhatever() when a select() on ClientSocket indicates
 * ready-for-reading (or when you want to block ).
 * Make sure not to eg. send txWhatever() then rxSomethingElse()
 * otherwise things will go downhill rather quickly for your bot.
 *
 * Apart from the input and output arguments being split between two
 * functions, the interface is the same as the non-tx/rx versions,
 * with the exception of rxGetField which needs to be passed a
 * pre-allocated buffer (TC_FIELD_SIZE bytes) to put the field into.
 */

extern int ClientSocket;

void txMoveBlock(int MoveCommand);
int rxMoveBlock(void);
void txUseSpecial(int PlayerNum);
int rxUseSpecial(void);
void txGetField(int PlayerNum);
char *rxGetField(char *field);
void txGetSpecials(void);
void rxGetSpecials(char *Specials);
void txGetCurrentBlock(void);
int rxGetCurrentBlock(int *BlockType, int *XPosition, int *YPosition);
void txGetNextBlock(void);
void rxGetNextBlock(int *BlockType);
void txGetMessage(void);
int rxGetMessage(int *FromPlayer, char **Message);
void txGetSpecialNotification(void);
int rxGetSpecialNotification(int *FromPlayer, int *ToPlayer, int *Special);

#endif /* __TC_CLIENT_H */
